Package: OrganismDbi
Title: Software to enable the smooth interfacing of different database
        packages
Description: The package enables a simple unified interface to several
	annotation packages each of which has its own schema by taking
	advantage of the fact that each of these packages implements a
	select methods.
Version: 1.54.0
Encoding: UTF-8
Authors@R: c(
	person("Marc", "Carlson", role = "aut"),
	person("Martin", "Morgan", role = "aut"),
	person("Valerie", "Obenchain", role = "aut"),
    	person("Aliyu Atiku Mustapha", role = "ctb",
                comment = "Converted 'OrganismDbi' vignette from Sweave to RMarkdown / HTML."),
        person("Bioconductor Package Maintainer", role = "cre",
                email = "maintainer@bioconductor.org"))
Depends: R (>= 2.14.0), BiocGenerics (>= 0.15.10), AnnotationDbi (>=
        1.33.15), Seqinfo, GenomicFeatures (>= 1.61.4)
Imports: methods, utils, stats, DBI, BiocManager, Biobase, graph, RBGL,
        S4Vectors, IRanges, GenomicRanges (>= 1.61.1)
Suggests: txdbmaker, GenomeInfoDbData, Homo.sapiens, Rattus.norvegicus,
        BSgenome.Hsapiens.UCSC.hg19, AnnotationHub, FDb.UCSC.tRNAs,
        rtracklayer, biomaRt, RUnit, RMariaDB, BiocStyle, knitr
Collate: AllGenerics.R AllClasses.R methods-select.R
        methods-transcripts.R createOrganismPackage.R seqinfo.R
        test_OrganismDbi_package.R
License: Artistic-2.0
biocViews: Annotation, Infrastructure
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/OrganismDbi
git_branch: RELEASE_3_23
git_last_commit: 3f6b0d3
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 23:43:48 UTC; biocbuild
Author: Marc Carlson [aut],
  Martin Morgan [aut],
  Valerie Obenchain [aut],
  Aliyu Atiku Mustapha [ctb] (Converted 'OrganismDbi' vignette from
    Sweave to RMarkdown / HTML.),
  Bioconductor Package Maintainer [cre]
Maintainer: Bioconductor Package Maintainer <maintainer@bioconductor.org>
