[<img src="https://www.bioconductor.org/images/logo/jpg/bioconductor_logo_rgb.jpg" width="200" align="right"/>](https://bioconductor.org/)

**OrganismDbi** is an R/Bioconductor software package that provides smooth interfacing of different annotation packages.

See https://bioconductor.org/packages/OrganismDbi for more information including how to install the release version of the package (please refrain from installing directly from GitHub).

